
include std/iup.e
include std/iupim.e
include std/iup_scintilla.e
include std/convert.e
include std/filesys.e
include std/machine.e
include std/sequence.e
include std/wildcard.e
include euphoria/keywords.e
include events.e

ifdef WINDOWS then
	constant DEFAULT_FONT = "Consolas"
elsedef
	constant DEFAULT_FONT = "Monospace"
end ifdef

constant KEYWORDS0 = allocate_string( stdseq:join(keywords, " "), 1 )
constant KEYWORDS1 = allocate_string( stdseq:join(builtins, " "), 1 )

--**
-- Add a new tab.
--
public function add_tab( sequence title = "Untitled", sequence text = "" )

	atom sci = IupScintilla()
	IupSetAttributes( sci, "TABSIZE=4, EXPAND=YES, VISIBLE=YES" )
	IupSetAttribute( sci, "TABTITLE", title )
	IupSetAttribute( sci, "TABIMAGE", get_image("IUP_FileNew") )

	atom tabs = IupGetHandle( "tabs" )
	IupAppend( tabs, sci )
	IupMap( sci )
	IupSetAttributeHandle( tabs, "VALUE", sci )
	IupSetFocus( sci )
	IupRefresh( tabs )

	IupSetAttribute( sci, "INSERT0", text )
	IupSetAttribute( sci, "UNDO", NULL )
	IupSetAttribute( sci, "SAVEDSTATE", NULL )
	IupSetCallback( sci, "SAVEPOINT_CB", Icallback("savepoint_cb") )

	IupSetCallback( sci, "K_cS", Icallback("save_cb") )

	return sci
end function

--**
-- Apply Scintilla lexer configuration based on extension.
--
public procedure apply_lexer( atom sci, sequence path = "" )

	IupSetAttribute( sci, "CLEARDOCUMENTSTYLE", "YES" )

	IupSetAttribute( sci, "STYLEFONT32", DEFAULT_FONT )
	IupSetAttribute( sci, "STYLEFONTSIZE32", "10" )
	IupSetAttribute( sci, "STYLECLEARALL", "" )

	sequence list = { "*.e", "*.ew", "*.ex", "*.exw" }
	if length( path ) = 0 or wildcard_any( list, path ) then

		-- FIXME: need to implement real Euphoria lexer!
		IupSetAttribute( sci, "LEXERLANGUAGE", "lua" )

		-- apply Euphoria keyword style
		IupSetAttribute( sci, "KEYWORDS0", KEYWORDS0 )
		IupSetAttribute( sci, "STYLEFGCOLOR5", "0 0 255" ) -- blue

		-- apply Euphoria builtin style
		IupSetAttribute( sci, "KEYWORDS1", KEYWORDS1 )
		IupSetAttribute( sci, "STYLEFGCOLOR13", "255 0 255" ) -- magenta

		-- apply Euphoria comment style
		IupSetAttribute( sci, "STYLEFGCOLOR2", "255 0 0" ) -- red
		IupSetAttribute( sci, "STYLEBOLD2", "YES" )

		-- apply Euphoria string style
		IupSetAttribute( sci, "STYLEFGCOLOR6", "0 128 0" ) -- green
		IupSetAttribute( sci, "STYLEFGCOLOR7", "0 128 0" ) -- green

		-- apply the style to the document
		IupSetAttribute( sci, "STARTSTYLING", "1" )
		IupSetAttribute( sci, "STYLING32", "0" )

	end if

end procedure

--**
-- Confirm the user wants to close.
--
public function confirm_close( sequence name )

	sequence message = sprintf( "Save changes to '%s'?", {name} )

	atom dlg = IupMessageDlg()
	IupSetAttribute( dlg, "TITLE", "Close" )
	IupSetAttribute( dlg, "VALUE", message )
	IupSetAttribute( dlg, "BUTTONS", "YESNOCANCEL" )
	IupSetAttribute( dlg, "DIALOGTYPE", "QUESTION" )

	IupPopup( dlg, IUP_CURRENT, IUP_CURRENT )
	atom response = IupGetInt( dlg, "BUTTONRESPONSE" )

	IupDetach( dlg )

	return response
end function

--**
-- Load our own custom image from disk.
--
public function get_image( sequence image )

	if length( image ) = 0 then
		return NULL
	end if

	-- replace IUP prefix
	if equal( image[1..4], "IUP_" ) then
		image = "PNG_" & image[5..$]
	end if

	-- check if the icon is already loaded
	atom handle = IupGetHandle( image )
	if handle != NULL then
		return image
	end if

	-- build the image path
	sequence path = stdseq:join(
		{"..","icons",image[5..$]&".png"}, SLASH)

	ifdef WINDOWS then

		-- use IM library to load image
		handle =  IupLoadImage( path )
		if handle = NULL then return NULL end if

		-- store the image handle
		IupSetHandle( image, handle )
		return image

	elsedef

		-- natively supports PNG images
		return path

	end ifdef

end function

--**
-- Match any of a list of wildcards.
--
public function wildcard_any( sequence list, sequence text )

	for i = 1 to length( list ) do
		if wildcard:is_match( list[i], text ) then
			return 1
		end if
	end for

	return 0
end function

--**
-- Get the global cursor position.
--
public function get_cursor_pos()

	sequence pos = IupGetGlobal( "CURSORPOS" )
	pos = stdseq:split( pos, "x" )

	atom x = to_integer( pos[1] )
	atom y = to_integer( pos[2] )

	return {x,y}
end function

--**
-- Toggle item state based on an attribute value.
--
public procedure toggle_item( sequence handle, object value )

	if sequence( value ) then
		value = length(value) > 0
			and equal(value, "NO") = 0
			and equal(value,  "0") = 0
	end if

	atom item = IupGetHandle( handle )
	atom state = IupGetInt( item, "ACTIVE" )
	
	if state != value  then
		IupSetInt( item, "ACTIVE", value )
	end if

end procedure

--**
-- Register hotkey commands.
--
public procedure register_hotkeys()

	atom dialog = IupGetHandle( "dialog" )

	-- File menu shortcuts
	IupSetCallback( dialog, "K_cN",  Icallback("new_cb") )			-- Ctrl-N
	IupSetCallback( dialog, "K_cO",  Icallback("open_cb") )			-- Ctrl-O
	IupSetCallback( dialog, "K_cS",  Icallback("save_cb") )			-- Ctrl-S
	IupSetCallback( dialog, "K_cW",  Icallback("close_cb") )		-- Ctrl-W

	-- Edit menu shortcuts (handled by Scintilla control)
--	IupSetCallback( dialog, "K_cZ",  Icallback("undo_cb") )			-- Ctrl-Z
--	IupSetCallback( dialog, "K_cY",  Icallback("redo_cb") )			-- Ctrl-Y
--	IupSetCallback( dialog, "K_cX",  Icallback("cut_cb") )			-- Ctrl-X
--	IupSetCallback( dialog, "K_cC",  Icallback("copy_cb") )			-- Ctrl-C
--	IupSetCallback( dialog, "K_cV",  Icallback("paste_cb") )		-- Ctrl+V
--	IupSetCallback( dialog, "K_cA",  Icallback("selectall_cb") )	-- Ctrl+A

	-- View menu shortcuts
	IupSetCallback( dialog, "K_cEqual", Icallback("zoomin_cb")    )	-- Ctrl++
	IupSetCallback( dialog, "K_cMinus", Icallback("zoomout_cb")   )	-- Ctrl+-
	IupSetCallback( dialog, "K_c0",     Icallback("zoomreset_cb") )	-- Ctrl+0

	-- Run menu shortcuts
	IupSetCallback( dialog, "K_F5",  Icallback("run_cb")  )			-- F5
	IupSetCallback( dialog, "K_F7",  Icallback("bind_cb") )			-- F7

	-- Help menu shortcuts
	IupSetCallback( dialog, "K_F12", Icallback("about_cb") )		-- F12

ifdef WINDOWS then
	-- Tab interface shortcuts
	IupSetCallback( dialog, "K_cPGUP", Icallback("pgup_cb") )		-- Ctrl+PgUp
	IupSetCallback( dialog, "K_cPGDN", Icallback("pgdn_cb") )		-- Ctrl+PgDn
end ifdef

end procedure

